SystemOrganization addCategory: #'Comet-Core'!
SystemOrganization addCategory: #'Comet-Testing'!

!WAKom class methodsFor: '*comet' stamp: 'lr 4/9/2006 20:52'!
service
	^ service! !

!Socket methodsFor: '*comet-sending' stamp: 'lr 4/10/2006 21:56'!
pushData: aStringOrByteArray
	"Send aStringOrByteArray to the remote host trough the given socket. Push the data immediately and ignore all kinds of errors. Return true if all the data could be sent successfully, in case of problems destroy the socket and return false."

	| bytesSent bytesToSend count |
	bytesSent := 0.
	bytesToSend := aStringOrByteArray size.
	[ bytesSent < bytesToSend ] whileTrue: [
		count := self 
			primSocket: socketHandle
			sendData: aStringOrByteArray
			startIndex: bytesSent + 1
			count: (bytesToSend - bytesSent min: 5000).
		bytesSent := bytesSent + count.
		(count = 0 and: [ bytesSent ~= bytesToSend ])
			ifTrue: [ self destroy. ^ false ] ].
	^ true! !

SUPrototype subclass: #CTComet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTComet methodsFor: 'private' stamp: 'lr 4/11/2006 22:00'!
defaultEffect
	^ 'Comet'! !

!CTComet methodsFor: 'private' stamp: 'lr 4/11/2006 00:48'!
defaultId
	^ nil! !

!CTComet methodsFor: 'private' stamp: 'lr 4/11/2006 22:00'!
type
	^ 'Ajax'! !

!CTComet methodsFor: 'options' stamp: 'lr 4/11/2006 07:23'!
url: aString
	self args: (Array with: aString)! !

WAScriptLibrary subclass: #CTLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTLibrary methodsFor: 'as yet unclassified' stamp: 'lr 4/11/2006 22:14'!
aBase
	^ '/* Seaside Comet API
 *  (c) 2006 Lukas Renggli (renggli@iam.unibe.ch)
 */
Ajax.Comet = Class.create();
Ajax.Comet.prototype = {
	initialize: function(url) {
		this.url = url;
		this.iframe = Builder.node("iframe", {
			style: "position: absolute; width: 0px; height: 0px; border-width: 0px;",
			onLoad: this.connect });
		document.body.appendChild(this.iframe);
		this.connect();
	},
	connect: function() {
		this.iframe.src = this.url;
	},
	disconnect: function() {
		this.iframe.src = "about:blank";
	}
};'! !

Object subclass: #CTNotifier
	instanceVariableNames: 'id mutex sockets process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

!CTNotifier class methodsFor: 'instance-creation' stamp: 'lr 4/10/2006 22:56'!
new
	^ CTRegistry default register: super new! !

!CTNotifier methodsFor: 'configuration' stamp: 'lr 4/10/2006 22:46'!
delay
	^ 7500! !

!CTNotifier methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:51'!
finalize
	super finalize.
	process terminate.
	sockets do: [ :each | each destroy ]! !

!CTNotifier methodsFor: 'private' stamp: 'lr 4/11/2006 00:38'!
header
	^ String streamContents: [ :stream |
		stream nextPutAll: '<?xml version="1.0" encoding="UTF-8"?><!!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"><html xmlns="http://www.w3.org/1999/xhtml"><head><meta name="generator" content="Seaside Comet" /></head><body>'.
		[ stream contents size < 1024 ]
			whileTrue: [ stream nextPut: Character space ] ]! !

!CTNotifier methodsFor: 'accessing' stamp: 'lr 4/9/2006 21:14'!
id
	^ id! !

!CTNotifier methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:41'!
initialize
	super initialize.
	id := (WAExternalID new: 24) asString.
	mutex := Semaphore forMutualExclusion.
	sockets := Array new.
	process := [ [ self process ] repeat ]
		forkAt: self priority
		named: self name! !

!CTNotifier methodsFor: 'actions' stamp: 'lr 4/11/2006 23:08'!
notify: anObject
	self send: '<script type="text/javascript">with(parent){' , anObject asJavascript , '}</script>'! !

!CTNotifier methodsFor: 'printing' stamp: 'lr 4/10/2006 17:15'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' id: '; print: self id! !

!CTNotifier methodsFor: 'configuration' stamp: 'lr 4/10/2006 22:55'!
priority
	^ 10! !

!CTNotifier methodsFor: 'private' stamp: 'lr 4/11/2006 00:26'!
process
	(Delay forMilliseconds: self delay)
		wait.
	self send: ' '! !

!CTNotifier methodsFor: 'actions' stamp: 'lr 4/11/2006 00:38'!
register: aSocket
	(aSocket pushData: (String streamContents: [ :stream |
		WAResponse new
			doNotCache;
			contents: self header;
			writeOn: stream ]))
		ifFalse: [ ^ self ].
	mutex critical: [ sockets := sockets copyWith: aSocket ]! !

!CTNotifier methodsFor: 'private' stamp: 'lr 4/10/2006 22:43'!
send: aString
	mutex critical: [
		sockets := sockets select: [ :each | 
			each pushData: aString ] ]! !

Object subclass: #CTRegistry
	instanceVariableNames: 'mutex notifiers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Core'!

CTRegistry class
	instanceVariableNames: 'running default'!

!CTRegistry class methodsFor: 'accessing' stamp: 'lr 4/10/2006 22:54'!
default
	^ default ifNil: [ default := self new ]! !

!CTRegistry class methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:57'!
initialize
	Smalltalk addToStartUpList: self after: WAKom! !

!CTRegistry class methodsFor: 'accessing' stamp: 'lr 4/11/2006 20:07'!
running
	^ running ifNil: [ running := false ]! !

!CTRegistry class methodsFor: 'actions' stamp: 'lr 4/11/2006 20:08'!
start
	| assembly |
	assembly := ModuleAssembly core.
	assembly alias: '/comet' to: [
		assembly addPlug: (MessageSend
			receiver: self default
			selector: #process:
			argument: nil) ].
	assembly alias: '/seaside' to: [
		assembly addPlug: (MessageSend
			receiver: WAKom default
			selector: #process:
			argument: nil) ].
	WAKom service
		plug: assembly rootModule.
	running := true! !

!CTRegistry class methodsFor: 'initialization' stamp: 'lr 4/11/2006 20:08'!
startUp
	self running ifNotNil: [ self start ]! !

!CTRegistry class methodsFor: 'actions' stamp: 'lr 4/11/2006 20:08'!
stop
	WAKom startOn: WAKom port.
	running := false! !

!CTRegistry methodsFor: 'initialization' stamp: 'lr 4/10/2006 22:51'!
initialize
	super initialize.
	mutex := Semaphore forMutualExclusion.
	notifiers := WeakValueDictionary new! !

!CTRegistry methodsFor: 'processing' stamp: 'lr 4/10/2006 22:52'!
process: aRequest
	| parts handler |
	parts := aRequest url findTokens: '/'.
 	handler := mutex critical: [
		(notifiers at: parts last ifAbsent: [ ^ nil ])
			ifNil: [ ^ nil ] ].
	handler register: aRequest stream socket.
	thisContext swapSender: nil.
	Processor activeProcess terminate! !

!CTRegistry methodsFor: 'actions' stamp: 'lr 4/10/2006 22:51'!
register: aNotifier
	^ mutex critical: [ notifiers at: aNotifier id put: aNotifier ]! !

!CTRegistry methodsFor: 'actions' stamp: 'lr 4/11/2006 20:15'!
unregister: aNotifier
	mutex critical: [ notifiers removeKey: aNotifier id ].
	aNotifier finalize! !

SUFunctionalTest subclass: #SUChatTest
	instanceVariableNames: 'username message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Comet-Testing'!

SUChatTest class
	instanceVariableNames: 'notifier'!

!SUChatTest class methodsFor: 'initialization' stamp: 'lr 4/11/2006 20:23'!
initialize
	(self registerAsApplication: 'chat') libraries 
		add: SUScriptLibrary;
		add: CTLibrary.
	notifier := CTNotifier new! !

!SUChatTest class methodsFor: 'accessing' stamp: 'lr 4/11/2006 20:23'!
notifier
	^ notifier! !

!SUChatTest methodsFor: 'private' stamp: 'lr 4/11/2006 23:17'!
event: aString
	self notifier notify: (SUInsertion new
		id: (SUJavascript on: 'messages');
		content: '<div class="message">' , 
			'<span class="username">' , username , '</span>' ,
			'<span class="event">' , aString , '</span>' ,
			'</div>')! !

!SUChatTest methodsFor: 'accessing' stamp: 'lr 4/11/2006 07:13'!
notifier
	^ self class notifier! !

!SUChatTest methodsFor: 'rendering' stamp: 'lr 4/11/2006 23:28'!
renderContentOn: html
	html textInput
		id: 'username';
		value: username;
		onBlur: (html request 
			triggerFormElement: 'username');
		callback: [ :value | username := value. self event: 'Joined' ].
	html div 
		id: 'messages';
		style: 'height: 200px; overflow: auto;'.
	html script: (CTComet new url: 'http://localhost:8080/comet/' , self notifier id).
	html textInput
		id: 'message';
		style: 'width: 100%';
		callback: [ :value | self send: value ];
		onEnter: (html request 
			triggerFormElement: 'message';
			onSuccess: (SUJavascript on: '$("message").value = ""'))! !

!SUChatTest methodsFor: 'private' stamp: 'lr 4/11/2006 23:17'!
send: aString
	self notifier notify: (SUInsertion new
		id: (SUJavascript on: 'messages');
		content: '<div class="message">' , 
			'<span class="username">' , username , '</span>' ,
			'<span class="message">' , aString , '</span>' ,
			'</div>')! !

!SUChatTest methodsFor: 'rendering' stamp: 'lr 4/11/2006 23:25'!
style
	^ 'div.message {
	margin-top: 3px;
	margin-bottom: 3px;
	background-color: #dddddd;
}
div.message span.username {
	margin-right: 1em;
	font-weight: bold;
}
div.message span.message {
}
div.message span.event {
	font-style: italic;
}
' 
! !
CTRegistry initialize!
SUChatTest initialize!
